{

=head1 NAME

Net::Google

=head1 SYNOPSIS

 use Net::Google;
 use constant LOCAL_GOOGLE_KEY => "********************************";

 my $google = Net::Google->new(key=>LOCAL_GOOGLE_KEY);
 my $search = $google->search();

 $search->query(qw(aaron cope));
 map { print $_->title()."\n"; } @{$search->results()};

=head1 DESCRIPTION

Provides a simple OOP-ish interface to the Google SOAP API

=cut

package Net::Google;
use strict;

use Carp;
use SOAP::Lite;

$Net::Google::VERSION = 0.1;

# This clever hack is courtesy Matt Webb:
# http://interconnected.org/home/more/GoogleSearch.pl.txt

# Redefine how the default deserializer handles booleans.
# Workaround because the 1999 schema implementation incorrectly doesn't
# accept "true" and "false" for boolean values.
# See http://groups.yahoo.com/group/soaplite/message/895

*SOAP::XMLSchema1999::Deserializer::as_boolean =
  *SOAP::XMLSchemaSOAP1_1::Deserializer::as_boolean = 
  \&SOAP::XMLSchema2001::Deserializer::as_boolean;

use constant SERVICES => {
			  "search" => "./Google/Services/GoogleSearch.wsdl",
			 };

=head1 Google methods

=head2 $google = Net::Google->new(%args)

=cut

sub new {
  my $pkg = shift;
  
  my $self = {};
  bless $self,$pkg;

  if (! $self->init(@_)) {
    return undef;
  }

  return $self;
}

sub init {
  my $self = shift;
  my $args = {@_};

  $self->{'_key'} = $args->{'key'};
  return 1;
}

sub _soap {
  my $service = shift;
  my $args    = {@_};

  my $soap = SOAP::Lite->service("file:".SERVICES->{$service});

  if ($args->{'debug'}) {
    $soap->on_debug(sub{print @_;});
  } 

  $soap->on_fault(sub{
		    my ($soap,$res) = @_; 
		    carp $res->faultstring();
		    return undef;
		  });

  return $soap;
}

=head2 $google->search(%args)

Returns a I<Search> object.

=cut

sub search {
  my $self = shift;
  my $args = {@_};

  my $debug = (defined($args->{'debug'})) ? $args->{debug} : $self->{'_debug'};
  return Search->new(&_soap("search",debug=>$debug),key=>$self->{'_key'});
}

=head1 Search methods

=cut

package Search;

sub new {
  my $pkg = shift;
  
  my $self = {};
  bless $self,$pkg;

  if (! $self->init(@_)) {
    return undef;
  }

  return $self;
}

sub init {
  my $self    = shift;
  my $service = shift;
  my $args    = {@_};

  $self->{'_service'} = $service;

  $self->{'_key'}         = $args->{'key'};
  $self->{'_query'}       = [];
  $self->{'_starts_at'}   = $args->{'starts_at'}   || 0;
  $self->{'_max_results'} = $args->{'max_results'} || 10;

  return 1;
}

=head2 $search->key()

=cut

sub key {
  my $self = shift;
  return $self->{'_key'};
}

=head2 $search->query(@data)

=cut

sub query {
  my $self = shift;
  my @data = @_;

  if ((scalar(@data) > 1) && ($data[0] eq "")) {
    $self->{'_query'} = [];
  }

  push @{$self->{'_query'}}, @data;
  return join(" ",@{$self->{'_query'}});
}

=head2 $search->starts_at($at)

=cut

sub starts_at {
  my $self = shift;
  my $at   = shift;

  if (defined($at)) {
    $self->{'_starts_at'} = $at;
  }

  return $self->{'_starts_at'};
}

=head2 $search->max_results($max)

=cut

sub max_results {
  my $self = shift;
  my $max  = shift;

  if (defined($max)) {
    $self->{'_max_results'} = $max;
  }

  return $self->{'_max_results'};
}

=head2 $search->filter($bool)

=cut

sub filter {
  my $self = shift;
  my $bool = shift;

  if (defined($bool)) {
    $self->{'_filter'} = ($bool) ? "true" : "false";;
  }

  return $self->{'_filter'};
}

=head2 $search->safe($bool)

=cut

sub safe {
  my $self = shift;
  my $bool = shift;

  if (defined($bool)) {
    $self->{'_safe'} = ($bool) ? "true" : "false";;
  }

  return $self->{'_safe'};
}

sub lr {}

sub ie {}

sub oe {}

=head2 $search->results()

Returns an array ref of I<Result> objects.

=cut

sub results {
  my $self = shift;
  my $result = $self->{'_service'}->doGoogleSearch(
						   $self->key(),
						   $self->query(),
						   $self->starts_at(),
						   $self->max_results(),
						   SOAP::Data->type(boolean =>$self->filter()),
						   "",
						   SOAP::Data->type(boolean =>$self->safe()),
						   "",
						   "latin1",
						   "latin1",
						  );

  if(ref($result->{resultElements}) eq "ARRAY") {
    map { $_ = Result->new($_); } @{$result->{resultElements}};
    return $result->{resultElements};
  }

  return [];
}

=head1 Result methods

=cut

package Result;
use vars qw ($AUTOLOAD);

sub new {
  my $pkg = shift;
  my $res = shift;
  return bless $res,$pkg;
}

=head2 $result->title()

=head2 $result->URL()

=head2 $result->snippet()

=head2 $result->cachedSize()

=head2 $result->directoryTitle()

=head2 $result->summary()

=head2 $result->hostName()

=head2 $result->directoryCategory()

=cut

sub AUTOLOAD {
  my $self = shift;
  $AUTOLOAD =~ s/.*:://;
  return $self->{$AUTOLOAD};
}

=head1 VERSION

0.1

=head1 DATE

$Date$

=head1 AUTHOR

Aaron Straup Cope

=head1 CAVEATS

There are still methods and functionality (passing default behaviours to child objects) that need to be implemented.

The package name may change.

=head1 LICENSE

Copyright (c) 2002, Aaron Straup Cope. All Rights Reserved.

This is free software, you may use it and distribute it under the same terms as Perl itself.

=cut

return 1;

}
